==> atm-0.14.tar.gz: atm/aqd/io.c <==
/* io.c - I/O operations */
 
/* Written 1996 by Werner Almesberger, EPFL-LRC */
 

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <linux/atm.h>
#include <linux/arequipa.h>

#include "atm.h"
#include "atmd.h"

#include "io.h"


#define COMPONENT "IO"


extern int pretty;

static int incoming;


void open_all(void)
{
    struct sockaddr_atmsvc addr;
    struct atm_blli blli;
    struct atm_qos qos;

    if ((incoming = socket(PF_ATMSVC,SOCK_DGRAM,ATM_AAL5)) < 0)
	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
    memset(&qos,0,sizeof(qos));
    qos.txtp.class = qos.rxtp.class = ATM_ANYCLASS;
	/* compatible with everything else */
    if (setsockopt(incoming,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0)
	diag(COMPONENT,DIAG_FATAL,"setsockopt SO_ATMQOS: %s",strerror(errno));
    memset(&addr,0,sizeof(addr));
    addr.sas_family = AF_ATMSVC;
    addr.sas_addr.bhli.hl_type = ATM_HL_USER;
    addr.sas_addr.bhli.hl_length = strlen("AREQ");
    strcpy(addr.sas_addr.bhli.hl_info,"AREQ");
    addr.sas_addr.blli = &blli;
    blli.l2_proto = ATM_L2_ISO8802;
    blli.l3_proto = ATM_L3_NONE;
    blli.next = NULL;
    if (bind(incoming,(struct sockaddr *) &addr,sizeof(addr)) < 0)
	diag(COMPONENT,DIAG_FATAL,"bind: %s",strerror(errno));
    if (listen(incoming,5) < 0)
	diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
}


void close_all(void)
{
    (void) close(incoming);
}


static void accept_new(void)
{
    char buffer[MAX_ATM_ADDR_LEN+1];
    struct sockaddr_atmsvc addr;
    int fd,len;

    len = sizeof(addr);
    if ((fd = accept(incoming,(struct sockaddr *) &addr,&len)) < 0) {
	diag(COMPONENT,errno == EUNATCH ? DIAG_FATAL : DIAG_ERROR,
	  "accept: %s",strerror(errno));
	return;
    }
    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
      0) strcpy(buffer,"<atm2text error>");
    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
    /*
     * Connection acceptance policy functions go here.
     */
    if (ioctl(fd,AREQUIPA_INCOMING,0) < 0)
        diag(COMPONENT,DIAG_ERROR,"ioctl AREQUIPA_INCOMING: %s",
	  strerror(errno));
    (void) close(fd); /* socket is now owned by the kernel */
}


void poll_loop(void)
{
    fd_set fds;
    int ret;

    while (1) {
	FD_ZERO(&fds);
	FD_SET(incoming,&fds);
	ret = select(incoming+1,&fds,NULL,NULL,NULL);
	if (ret < 0) {
	    if (errno != EINTR) perror("select");
	}
	else if (FD_ISSET(incoming,&fds)) accept_new();
		/* why else would we get here ? */
    }
}

==> atm-0.15.tar.gz: atm/aqd/io.c <==
/* io.c - I/O operations */
 
/* Written 1996 by Werner Almesberger, EPFL-LRC */
 

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <linux/atm.h>
#include <linux/arequipa.h>

#include "atm.h"
#include "atmd.h"

#include "io.h"


#define COMPONENT "IO"


extern int pretty;

static int kernel,incoming;


void open_all(void)
{
    struct sockaddr_atmsvc addr;
    struct atm_blli blli;
    struct atm_qos qos;

    if ((kernel = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0)
	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
    if (ioctl(kernel,AREQUIPA_CTRL,0) < 0)
	diag(COMPONENT,DIAG_FATAL,"ioctl AREQUIPA_CTRL: %s",strerror(errno));
    if ((incoming = socket(PF_ATMSVC,SOCK_DGRAM,ATM_AAL5)) < 0)
	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
    memset(&qos,0,sizeof(qos));
    qos.txtp.class = qos.rxtp.class = ATM_ANYCLASS;
	/* compatible with everything else */
    if (setsockopt(incoming,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0)
	diag(COMPONENT,DIAG_FATAL,"setsockopt SO_ATMQOS: %s",strerror(errno));
    memset(&addr,0,sizeof(addr));
    addr.sas_family = AF_ATMSVC;
    addr.sas_addr.bhli.hl_type = ATM_HL_USER;
    addr.sas_addr.bhli.hl_length = strlen("AREQ");
    strcpy(addr.sas_addr.bhli.hl_info,"AREQ");
    addr.sas_addr.blli = &blli;
    blli.l2_proto = ATM_L2_ISO8802;
    blli.l3_proto = ATM_L3_NONE;
    blli.next = NULL;
    if (bind(incoming,(struct sockaddr *) &addr,sizeof(addr)) < 0)
	diag(COMPONENT,DIAG_FATAL,"bind: %s",strerror(errno));
    if (listen(incoming,5) < 0)
	diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
}


void close_all(void)
{
    (void) close(kernel);
    (void) close(incoming);
}


static void recv_kernel(void)
{
    struct atm_vcc *vcc;
    int size;

    size = read(kernel,&vcc,sizeof(vcc));
    if (size < 0) {
	diag(COMPONENT,DIAG_ERROR,"read kernel: %s",strerror(errno));
	return;
    }
    if (size != sizeof(vcc))
	diag(COMPONENT,DIAG_FATAL,"kernel message has bad size (%d != %d)",
	  size,sizeof(vcc));
    diag(COMPONENT,DIAG_DEBUG,"Closing socket 0x%08lx",(unsigned long) vcc);
    if (ioctl(kernel,AREQUIPA_CLS3RD,(unsigned long) vcc) < 0)
	diag(COMPONENT,DIAG_ERROR,"ioctl AREQUIPA_CLS3RD: %s",strerror(errno));
}


static void accept_new(void)
{
    char buffer[MAX_ATM_ADDR_LEN+1];
    struct sockaddr_atmsvc addr;
    int fd,len;

    len = sizeof(addr);
    if ((fd = accept(incoming,(struct sockaddr *) &addr,&len)) < 0) {
	diag(COMPONENT,errno == EUNATCH ? DIAG_FATAL : DIAG_ERROR,
	  "accept: %s",strerror(errno));
	return;
    }
    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
      0) strcpy(buffer,"<atm2text error>");
    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);

    /*
     * Connection acceptance policy functions go here.
     */

    if (ioctl(fd,AREQUIPA_INCOMING,0) < 0)
        diag(COMPONENT,DIAG_ERROR,"ioctl AREQUIPA_INCOMING: %s",
	  strerror(errno));
    (void) close(fd); /* socket is now owned by the kernel */
}


void poll_loop(void)
{
    fd_set fds;
    int ret;

    while (1) {
	FD_ZERO(&fds);
	FD_SET(kernel,&fds);
	FD_SET(incoming,&fds);
	ret = select((kernel > incoming ? kernel : incoming)+1,&fds,NULL,NULL,
	  NULL);
	if (ret < 0) {
	    if (errno != EINTR) perror("select");
	    continue;
	}
	if (FD_ISSET(kernel,&fds)) recv_kernel();
	if (FD_ISSET(incoming,&fds)) accept_new();
	
    }
}
